#pragma once

#include <sys/queue.h>

#include "scene_common.h"

enum kysec_exectl_strategy_type {
    /* 执行控制机是制基于签名*/
    KYSEC_SIGNATURE_STRATEGY,
    /* 执行控制机制基于度量(短码+长码)*/
    KYSEC_MEASURE_STRATEGY,
};

enum kysec_exectl_temporaryctl_strategy_type {
    /*
    两个功能互斥
    软件包安装时创建的临时文件被记录上create id
    打开此策略,代表仅放行软件包执行自己的临时文件
    */
    KYSEC_PACKAGE_TMPFILE_STRATEGY,

    /*
    设置此策略,代表所有临时文件都可以被执行
    */
    KYSEC_ALL_TMPFILE_STRATEGY,
};

struct kysec_scene_exectl_obj {
    char *fpath;  // 文件路径
    char *hash;   // 摘要值
    int ftype;    // 文件类型
    int dtype;    // 数据类型 (系统原生/后续新增的)
    int status;   // 数据状态  0正常 1被篡改
};

struct kysec_scene_exectl_obj_entry {
    struct kysec_scene_exectl_obj obj;
    SLIST_ENTRY(kysec_scene_exectl_obj_entry)
    entries;
};
/**
 * @brief 执行控制单链表结构体
 *
 * 单链表结构体名字是kysec_exectl_obj_list，链表成员类型是kysec_exectl_obj_entry
 */
SLIST_HEAD(kysec_scene_exectl_obj_list, kysec_scene_exectl_obj_entry);

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief 添加文件或目录的执行特权(完全不管控)
 * @param path 文件或目录的绝对路径
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_exectl_add_file_in_permissive(const char *path);

/**
 * @brief 删除文件或目录到执行特权(完全不管控)
 * @param path 文件或目录的绝对路径
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_exectl_remove_file_from_permissive(const char *path);

/**
 * @brief 获取不管控文件的策略
 * @param count 个数
 * @return 字符串数组  NULL:无数据
 *         调用者使用完需要手动释放内存
 */
char **kysec_scene_exectl_get_policy_for_file_permissive(int *count);

/**
 * @brief 添加软件包的执行特权(完全不管控)
 * @param package 包名称
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_exectl_add_package_in_permissive(const char *package);

/**
 * @brief 删除软件包到执行特权(完全不管控)
 * @param package 包名称
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_exectl_remove_package_from_permissive(const char *package);

/**
 * @brief 获取不管控软件包的策略
 * @param count 个数
 * @return 字符串数组  NULL:无数据
 *         调用者使用完需要手动释放内存
 */
char **kysec_scene_exectl_get_policy_for_package_permissive(int *count);

/**
 * @brief 添加排除的文件或目录(对于完全不管控的目录下某些文件还需要管控特例)
 * @details 某个目录不管控,但目录下个别文件需要被管控,将文件从不管控中排除
 * @param path 文件或目录的绝对路径
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_exectl_add_exclude_file_for_permissive(const char *path);

/**
 * @brief 删除排除的文件或目录(对于完全不管控的目录下某些文件还需要管控特例)
 * @param path 文件或目录的绝对路径
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_exectl_remove_exclude_file_from_permissive(const char *path);

/**
 * @brief 获取排除的文件或目录的策略
 * @param count 个数
 * @return 字符串数组  NULL:无数据
 *         调用者使用完需要手动释放内存
 */
char **kysec_scene_exectl_get_exclude_file_policy_from_permissive(int *count);

/**
 * @brief 设置exectl机制 (基于验签或基于度量)
 *        若设置了验签为开启，则自动关闭度量,
 * 同时仅一个功能开启(桌面可以设置验签，服务器默认只有度量机制)
 * @param strategy 机制类型
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_exectl_set_strategy(enum kysec_exectl_strategy_type strategy);

/**
 * @brief 获取执行控制机制状态
 * @details 若验签是开启的 则度量就是关闭的,反之亦然,同时仅一个功能开启
 *        所以不再单独提供一个查询当前exectl的机制接口
 * @return 0:签名 1:度量 其他:获取失败
 *
 */
int kysec_scene_exectl_get_strategy();

/**
 * @brief 设置临时程序管控机制
 * @details 两种机制
 *          1.仅软件包能够执行自己创建的临时文件(默认状态)
 *          2.允许所有临时文件执行
 * @param strategy 机制类型
 * @return 0:设置成功 其他:设置失败
 */
int kysec_scene_exectl_set_temporaryctl_strategy(
    enum kysec_exectl_temporaryctl_strategy_type strategy);

/**
 * @brief 获取临时程序管控机制
 * @param strategy 机制类型
 * @return  0:仅软件包创建的临时文件 1:所有临时文件 其他:获取失败
 */
int kysec_scene_exectl_get_temporaryctl_strategy();

/**
 * @brief 设置文件的执行控制扩展属性
 * @param path 文件路径
 * @return 0:成功 其他:失败
 */
int kysec_scene_exectl_xattr_set(const char *path);

/**
 * @brief 获取文件的执行控制扩展属性
 * @param path 文件路径
 * @param label 存储获取到的标记字符串
 * @return 成功返回字符串长度, 其他 失败
 */
int kysec_scene_exectl_xattr_get(const char *path, char **label);

/**
 * @brief 批量设置文件的执行控制扩展属性
 * @param paths 文件路径数组
 * @param count 数组大小
 * @return int >0 设置成功的数量，其他 失败
 */
int kysec_scene_exectl_xattr_multi_set(char *paths[], size_t count);

/**
 * @brief 遍历目录，设置文件的执行控制扩展属性
 * @param dir 文件路径
 * @return int 0 成功，其他 失败
 */

int kysec_scene_exectl_xattr_set_for_dir(const char *dir);

/**
 * @brief 将文件的执行控制扩展清零
 * @param path 文件路径
 * @return int 0 成功，其他 失败
 */
int kysec_scene_exectl_xattr_clear(const char *path);

/**
 * @brief 批量清除文件的执行控制扩展属性
 * @param paths 文件路径数组
 * @param count 数组大小
 * @return int >0 清除成功的数量，其他 失败
 */
int kysec_scene_exectl_xattr_multi_clear(char *paths[], size_t count);

/**
 * @brief 遍历目录，将文件的执行控制扩展清零
 * @param dir 文件路径
 * @return int 0 成功，其他 失败
 */
int kysec_scene_exectl_xattr_clear_for_dir(const char *dir);

/**
 * @brief 根据路径添加白名单记录
 * @param path 文件路径
 * @return 0:成功 其他:失败
 */
int kysec_scene_exectl_wlist_add(const char *path);

/**
 * @brief 根据路径批量添加白名单记录
 * @param paths 文件路径数组
 * @param count 数组大小
 * @return int >0 设置成功的数量，其他 失败
 */
int kysec_scene_exectl_wlist_multi_add(char *paths[], size_t count);

/**
 * @brief 根据目录递归添加白名单记录
 * @param dir 目录路径
 * @return 0:成功 其他:失败
 */
int kysec_scene_exectl_wlist_add_by_dir(const char *dir);

/**
 * @brief 根据路径更新白名单记录
 * @param path 文件路径
 * @return 0:成功 其他:失败
 */
int kysec_scene_exectl_wlist_update(const char *path);

/**
 * @brief 根据路径批量更新白名单记录
 * @param paths 文件路径数组
 * @param count 数组大小
 * @return int >0 设置成功的数量，其他 失败
 */
int kysec_scene_exectl_wlist_multi_update(char *paths[], size_t count);

/**
 * @brief 根据目录递归更新白名单记录
 * @param dir 目录路径
 * @return 0:成功 其他:失败
 */
int kysec_scene_exectl_wlist_update_by_dir(const char *dir);

/**
 * @brief 根据路径删除白名单记录
 * @param path 文件路径
 * @return 0:成功 其他:失败
 */
int kysec_scene_exectl_wlist_delete(const char *path);

/**
 * @brief 根据路径批量删除白名单记录
 * @param paths 文件路径数组
 * @param count 数组大小
 * @return int >0 设置成功的数量，其他 失败
 */
int kysec_scene_exectl_wlist_multi_delete(char *paths[], size_t count);

/**
 * @brief 根据目录递归删除白名单记录
 * @param dir 目录路径
 * @return 0:成功 其他:失败
 */
int kysec_scene_exectl_wlist_delete_by_dir(const char *dir);

/**
 * @brief 获取执行控制白名单数据
 * @param objs 链表
 * @return 0:成功 其他:失败
 */
int kysec_scene_exectl_wlist_load_all(struct kysec_scene_exectl_obj_list *objs);

/**
 * @brief 获取执行控制白名单数据
 * @param key 关键字
 * @param objs 链表
 * @return 0:成功 其他:失败
 */
int kysec_scene_exectl_wlist_load(const char *key,
                                  struct kysec_scene_exectl_obj_list *objs);

/**
 * @brief 根据路径获取执行控制白名单数据
 * @param path 文件路径
 * @param obj 节点数据   需要调用者使用完通过kysec_scene_exectl_obj_free释放内存
 * @return 0:成功 其他:失败
 */
int kysec_scene_exectl_wlist_load_by_path(const char *path,
                                          struct kysec_scene_exectl_obj *obj);

/**
 * @brief 根据路径获取hash值
 * @param path 文件路径
 * @return hash字符串
 */
char *kysec_scene_exectl_wlist_get_hash_by_path(const char *path);

/**
 * @brief 修改数据库对于文件的数据项的 status 字段
 * @param path 要更新的文件的路径
 * @param status 执行控制保护状态：0 正常，1 已篡改
 * @return 0:成功 其他:失败
 */
int kysec_scene_exectl_wlist_change_status(const char *path, int status);

/**
 * @brief 批量修改数据库对于文件的数据项的 status 字段
 * @param paths 要删除的文件的路径数组
 * @param count 路径个数
 * @param status 执行控制保护状态：0 正常，1 已篡改
 * @return 0:成功 其他:失败
 */
int kysec_scene_exectl_wlist_multi_change_status(char *paths[], size_t count,
                                                 int status);
/**
 * @brief 释放链表内存
 * @param objs 链表
 *
 */
void kysec_scene_exectl_objs_free(struct kysec_scene_exectl_obj_list *objs);

/**
 * @brief 释放obj内存
 * @param obj
 *
 */
void kysec_scene_exectl_obj_free(struct kysec_scene_exectl_obj *obj);

#ifdef __cplusplus
}
#endif
