#pragma once

#include <stdint.h>

#define KYSEC_SCENE_KID_LEN 128
#define KYSEC_SCENE_KID_PATH_LEN 4096

/**
 * @brief kid策略中主体和客体的类型枚举
 */
enum KYSEC_SCENE_KID_INSTANCE_TYPE {
  /**
   * @brief 文件，相应的要填写文件的绝对路径
   */
  KYSEC_SCENE_KID_TYPE_FILE = 1,

  /**
   * @brief 软件包，相应的要填写软件包的名称
   */
  KYSEC_SCENE_KID_TYPE_PACKAGE,
};

/**
 * @brief kid策略，每一个实例代表一条kid策略
 */
struct kysec_scene_kid_policy {
  /**
   * @brief 主体类型，取KYSEC_SCENE_KID_INSTANCE_TYPE类型
   */
  int subject_type;

  /**
   * @brief 主体的软件包名或文件绝对路径，需要和subject_type对应
   */
  char subject[4096];

  /**
   * @brief 客体的绝对路径
   */
  char object[4096];

  /**
   * @brief 权限
   * @detail 0x11111分别为执行、重命名、删除、写、读
   */
  int32_t perm;
};

/**
 * @brief kid策略，每一个实例代表一条kid主体
 */
struct kysec_scene_kid_subject {
  /**
   * @brief 主体类型，取KYSEC_SCENE_KID_INSTANCE_TYPE类型
   * @detail 判断两个kid主体是否相同时会使用该字段
   */
  int subject_type;

  /**
   * @brief 主体的软件包名或文件绝对路径，需要和subject_type对应
   * @detail 判断两个kid主体是否相同时会使用该字段
   */
  char subject[4096];

  /**
   * @brief 可读的，展示给用户的名称
   * @detail 该字段仅用来保存名称，不会用于判断两个kid主体是否相同
   */
  char readable_name[128];
};

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief 添加主体应用
 * @param subject[in] 主体
 * @return 0:成功 其他:失败，取KYSEC_SCENE_CALL_RESULT中的值
 */
int kysec_scene_add_kid_subject(const struct kysec_scene_kid_subject *subject);

/**
 * @brief 根据软件包名称或者文件路径删除应用
 * @param path[in] 软件包名或者文件路径，需要和subject_type对应
 * @param subject_type[in] 主体类型，取KYSEC_SCENE_KID_INSTANCE_TYPE类型
 * @return 0:成功 其他:失败，取KYSEC_SCENE_CALL_RESULT中的值
 */
int kysec_scene_remove_kid_subject(const char *path, int subject_type);

/**
 * @brief 查询当前所有的kid应用
 * @param count[out]
 * @return 非NULL：成功，返回所有的kid应用
 *         NULL且count==0：成功，当前没有任何kid应用
 *         NULL且count<0：失败
 */
struct kysec_scene_kid_subject *kysec_scene_get_kid_subject(int *count);

/**
 * @brief 根据文件夹路径添加隐私资源
 * @param path[in] 绝对路径，必须是$HOME/xxx，即家目录下的一级目录
 * @return 0:成功 其他:失败，取KYSEC_SCENE_CALL_RESULT中的值
 */
int kysec_scene_add_kid_resource(const char *path);

/**
 * @brief 根据文件夹路径删除隐私资源
 * @param path[in] 绝对路径，必须是$HOME/xxx，即家目录下的一级目录
 * @return 0:成功 其他:失败，取KYSEC_SCENE_CALL_RESULT中的值
 */
int kysec_scene_remove_kid_resource(const char *path);

/**
 * @brief 获取当前所有的kid隐私资源的路径
 * @param count[out] 返回的路径的数目
 * @return 非NULL：成功，返回所有的路径
 *         NULL且count==0：成功，当前没有任何隐私资源
 *         NULL且count<0：失败
 */
char **kysec_scene_get_kid_resource(int *count);

/**
 * @brief 添加kid策略
 * @param subject[in] 主体
 * @param resource[in] 隐私资源的绝对路径
 * @param perm[in] 权限，0x11111为执行、重命名、删除、写、读
 * @return 0:成功 其他:失败，取KYSEC_SCENE_CALL_RESULT中的值
 */
int kysec_scene_add_kid_policy(struct kysec_scene_kid_subject *subject,
                               const char *resource, int perm);

/**
 * @brief 删除kid策略
 * @param subject[in] 主体
 * @param resource[in] 隐私资源的绝对路径
 * @return 0:成功 其他:失败，取KYSEC_SCENE_CALL_RESULT中的值
 */
int kysec_scene_remove_kid_policy(const struct kysec_scene_kid_subject *subject,
                                  const char *resource);

/**
 * @brief 查询所有的kid策略
 * @param count[out] 策略数量
 * @return 非NULL：成功，返回所有的策略
 *         NULL且count==0：成功，当前主体和资源之间没有kid策略
 *         NULL且count<0：失败
 */
struct kysec_scene_kid_policy *kysec_scene_get_kid_policy(int *count);

/**
 * @brief 获取某个subject的所有kid策略
 * @param subject[in] 主体
 * @param count[out] 策略数量
 * @return 非NULL：成功，返回所有的策略
 *         NULL且count==0：成功，当前没有任何kid策略
 *         NULL且count<0：失败
 */
struct kysec_scene_kid_policy *kysec_scene_get_kid_policy_by_subject(
    const struct kysec_scene_kid_subject *subject, int *count);

#ifdef __cplusplus
}
#endif
